<?php

namespace apexl\Io\modules\user\entities\operators;

use apexl\Io\exceptions\RecordNotFoundException;
use apexl\Io\operators\entityDatabaseOperator;

class refreshTokenOperator extends entityDatabaseOperator
{
    /**
     * @throws RecordNotFoundException
     */
    public function loadByToken($token): array
    {
        return $this->loadBy('token', $token);
    }
}
